/*
 * jabak Java Backup
 * 
 * Copyright (C) under the terms of the GNU General Public License
 * as explained in the file "disclaimer.txt"
 */

package backup;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * main backup routines for backing up a single file.
 * set up as an objectHandler to enable working
 * with Util.recurseDirectory
 * 
 * @author  Georg Dietrich
 * @version 1.3 on 2004-05-05
 * @version 1.2 on 2004-05-03
 * @version 1.1 on 2004-04-14
 * @version 1.0 on 2004-04-09
 */
public class Backup implements ObjectHandler {

	/** the collected backup settings */
	protected Settings settings;

	/**
	 * constructor saves the settings object for further reference
	 * 
	 * @param mySettings  the backup settings to use
	 */
	public Backup (Settings mySettings) {
		settings = mySettings;
	}
	
	/**
	 * main backup routine, gets called for all files which might be backed up.
	 * returns false as soon as the "don't continue" flag is set.
	 * 
	 * @param object  the single file to back up
	 */
	public boolean perform(Object oSource) {

		// check whether "don't continue" flag is set
		if (!settings.backupFinished) return false;
		
		// get the files needed
		File source  = (File) oSource;
		File backup  = Util.source2backup(source, settings);
		File data    = Util.source2data(source, settings);

		// eventually generate current file's hash sum
		long hashSum = settings.useHash ? Util.hashSum(source) : -1;
		// get the current file's modification date
		long modDate = source.lastModified();
		
		// scan the data file
		DataFile dataFile = new DataFile(data);
		
		// if data file does not exist or has length of 0, backup
		if (data.length() == 0L) {
			backup (source, backup, dataFile, hashSum, modDate);
			return true;
		}

		// if backup conditions apply, back up and return.
		if (dataFile.checkState(settings, hashSum, modDate)) {
			backup (source, backup, dataFile, hashSum, modDate);
			return true;
		}		

		// increase number of skipped files and update ui
		settings.filesSkipped++;
		settings.log.display(settings);
		settings.log.log ("skipped "+backup, 3);
		return true;
	}

	/**
	 * backup one file to zip
	 * 
	 * @param source  the source file to back up
	 * @param backup  the target file to write into
	 * @param data    the data file to write status info
	 */
	void backup (File source, File backup, DataFile dataFile, long hashSum, long modDate) {

		// check whether backup size limit is reached. YES -> set flag and don't backup.
		if (settings.currentSize*settings.sizeTolerance >= settings.backupSize) { settings.backupFinished = false; return; } 

		// assure the necessary directories are present
		Util.makeDirs(backup);

		// write the zip file
		try {
			
			// initialize the zip output stream
			FileOutputStream fileout = new FileOutputStream(backup);
			ZipOutputStream  zipout  = new ZipOutputStream (fileout);

			// initialize zip entry
			String sourceName = source.getName();
			ZipEntry entry = new ZipEntry(sourceName);
			entry.setTime(source.lastModified());
			zipout.putNextEntry(entry);

			// initialize zip compression level
			int zipLevel = settings.defaultZipValue;
			for (int i = 0; i < settings.zipFilters.size(); i++) {
				if (sourceName.matches(settings.zipFilters.get(i).toString()))
					zipLevel = Util.parseInt(settings.zipValues.get(i).toString());
			}
			zipout.setLevel(zipLevel);
			
			// initialize source file input stream
			FileInputStream  filein  = new FileInputStream (source);
			byte[]           buffer  = new byte[1024];

			// copy source to target
			for (int read = filein.read(buffer); read != -1; read = filein.read(buffer)) {
				zipout.write(buffer,0,read);
			}

			// close target and source
			zipout.closeEntry();
			zipout.close();
			fileout.close();
			filein.close();

		} catch (Exception e) {
			System.err.println ("problem during backup of file "+source);
			e.printStackTrace();
			throw new RuntimeException();
		}
		
		// add the zip file's length to the current length
		settings.currentSize += backup.length();

		// check whether backup size limit plus tolerance is overstepped
		if (settings.currentSize >= settings.backupSize*settings.sizeTolerance)
		{
			// check whether more than one file was backed up
			if (settings.currentSize > backup.length()) {
				// yes: delete last zip file and set the don't continue flag
				backup.delete();
				settings.log.log("deleted last file "+backup+" due to overstepping size limit.",2);
				settings.backupFinished = false;
				return;
			}
		} 
		
		// store updates in data file
		switch (settings.backupType) {
			case 4:
			dataFile.fulHash = hashSum;
			dataFile.fulMod  = modDate;
			dataFile.fulDate = settings.timeStamp;
			case 3:
			case 2:
			dataFile.difHash = hashSum;
			dataFile.difMod  = modDate;
			dataFile.difDate = settings.timeStamp;
			case 1:
			dataFile.incHash = hashSum;
			dataFile.incMod  = modDate;
			dataFile.incDate = settings.timeStamp;
			dataFile.save(settings);
			break;
			
			default:
			throw new RuntimeException();
		}
		
		// increase number of files backed up and update ui
		settings.filesBackedUp++;
		settings.log.display(settings);
		settings.log.log (settings.currentSize+" of "+settings.backupSize+" with "+backup, 2);
	}
}
