/*
 * jabak Java Backup
 * 
 * Copyright (C) under the terms of the GNU General Public License
 * as explained in the file "disclaimer.txt"
 */

package backup;

import java.io.File;
import java.util.Arrays;
import java.util.Vector;

import ui.GuiSettings;
import ui.Local;
import ui.Log;
import ui.StringContainer;

/**
 * file to zip backup core start class.  types of backup supported are
 * incremental - differential2differential - differential2full - full.
 * 
 * @author  Georg Dietrich
 * @version 2.0 on 2004-04-25
 * @version 1.3 on 2004-04-13
 * @version 1.2 on 2004-04-10
 * @version 1.0 on 2004-04-08
 */
public class CoreMain {

	public CoreMain(GuiSettings guiSettings) {

		// translate GuiSettings to CoreSettings
		Settings settings = new Settings();

		// initialize backup file paths
		settings.paths = new Paths(guiSettings);

		// initialize other settings
		settings.backupSize      = ((long)Util.parseInt(guiSettings.size))*1000000;
		settings.backupType      = Util.parseInt(guiSettings.backupType);
		settings.useHash         = guiSettings.compare.toString().equals("2") | guiSettings.compare.toString().equals("3");
		settings.useDate         = guiSettings.compare.toString().equals("1") | guiSettings.compare.toString().equals("2");
		settings.filterOut       = scanFilters(guiSettings.filters, settings.fileFilters); // also sets settings.filter
		settings.filterString    = guiSettings.filters.toString();
		settings.defaultZipValue = scanZipFilters(guiSettings.zipFilters, settings.zipFilters, settings.zipValues);
		settings.log             = guiSettings.log;

		// read status of last backups
		settings.dependencies    = new Dependencies (settings.paths.dataDesc);
		// get the new time stamp
		String message           = settings.dependencies.setTimeStamp (settings);
		// if a message is transmitted, don't proceed
		if (message != "") {
			guiSettings.result = message;
			return;
		}
		
		// add backup time stamp to backup path
		settings.paths.adaptBackupPath(settings.timeStamp);

		// create backup files
		Util.recurseDirectoryFilter(new File(settings.paths.sourceDir), new Backup(settings), settings);

		// if no data were saved, quit
		if (settings.currentSize == 0) {
			guiSettings.result = "Nothing to do: it seems to me all files are already\n"+
			                     "taken care of in earlier backups";
			return;
		}

		// clean up
		if (settings.backupFinished) {

			String type = "_"+Local.BACKUP_TYPE_NAME[settings.backupType];

			// add backup type to time stamp
			settings.timeStamp += type;

			// rename backup directory
			if (new File(settings.paths.backupDir).renameTo(new File(settings.paths.backupDir+type))) {
				// put new time stamp in the backup paths
				settings.paths.adaptBackupPath(settings.timeStamp);
			} else {
				Log.println("Could not rename backup root to "+settings.timeStamp);
			}

			
			// check backup data entries for orphan files
			Util.recurseDirectory(new File(settings.paths.dataDir), new DataCheck(settings));

			// generate filelist.zip file
			new FileList(new File(settings.paths.backupList),settings);

			// store result message
			guiSettings.result = "Backup complete.\n";
			
		} else {
			
			// store result message
			guiSettings.result = "Backup reached size limit.  Start backup again\n"+
					 							   "with the same parameters to cover the rest.";
		}

		// update backup dependency chain and file
		settings.dependencies.set(settings.paths.dataDir,settings);

		// save a human readable description of this backup in the backup directory
		new BackupDescription (new File(settings.paths.backupDesc),settings);
		
	}
	
	/**
	 * scan a source filter string into a target vector field with all
	 * necessary adaptions of data
	 * 
	 * @param source  the source string
	 * @param target  the target vector
	 * @return        true if negative filtering is detected
	 */
	protected boolean scanFilters (StringContainer source, Vector target) {
		target.addAll(Arrays.asList(source.toString().split(" ")));
		// detect empty filters == use all files
		if (target.size() == 0) {
			return true;
		}
		// detect single "all" filter
		if (target.size() == 1 && target.get(0).equals("all")) {
			target.remove(0);
			return true;
		}
		// detect "not" keyword
		if (target.get(0).equals("not")) {
			target.remove(0);
			return true;
		}
		// return the list of positive filters
		return false;
	}

	/**
	 * scan a source zip filter string into the two  target vector fields with all
	 * necessary adaptions of data
	 * 
	 * @param source  the source string
	 * @param target  the target vector
	 * @param value   the target compression values
	 * @return        the default compression value
	 */
	protected int scanZipFilters (StringContainer source, Vector target, Vector value) {
		int defaultValue = 9;
		target.addAll(Arrays.asList(source.toString().split(" ")));
		// detect empty filters == return default value
		if (target.size() == 0) {
			return defaultValue;
		}
		// detect single value filter
		if (target.size() == 1) {
			return Util.parseInt(target.remove(0).toString());
		}
		// scan all filters
		for (int i = 0; i < target.size(); i++) {
			if (target.get(i).equals("default")) {
				// default key word found: remove "default" word
				target.remove(i);
				// remove default value
				defaultValue = Util.parseInt((String)target.remove(i));
				// update counter
				i--;
			} else {
				// ordinary filter: move filter value to value vector
				value.add(target.remove(i+1));
			}
		}
		return defaultValue;
	}
	
}
