/*
 * jabak Java Backup
 * 
 * Copyright (C) under the terms of the GNU General Public License
 * as explained in the file "disclaimer.txt"
 */

package backup;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.text.DecimalFormat;
import java.util.Enumeration;
import java.util.Vector;

import ui.Log;

/**
 * @author  Georg Dietrich
 * @version 1.2 on 2004-05-01
 * @version 1.0 on 2004-04-11
 */
public class Dependencies {

	/** the full backups this one is dependent of */
	Vector fullDep = new Vector();
	/** the diff backup this one is dependent of */
	Vector diffDep = new Vector();
	/** the incr backup this one is dependent of */
	Vector incrDep = new Vector();
	/** the list of earlier backups */
	Vector list = new Vector();
	/** whether the last backup was finished */
	public boolean finished = true;
	/** which type the last backup was of */
	public int type = 0;
	/** the time stamps of the last backup set */
	public Vector timeStamps = new Vector();

	public Dependencies (String path) {

		File file = new File(path);
		
		// parse file
		if (file.exists()) {
			try {
				BufferedReader reader = new BufferedReader(new FileReader(file));
				
				if (!reader.readLine().equals("gb_1.1")) { 
					Log.println("conflicting versions in data file "+file);
					Log.println("remove file and try again.");
					throw new RuntimeException();
				}
				reader.readLine(); // "."
				
				reader.readLine(); // "status, type, and, if incomplete, time stamps of the last backup set"
				finished = reader.readLine().equals("complete");
				type = Util.parseInt(reader.readLine());
				for (String string = reader.readLine(); !string.equals("."); string = reader.readLine()) {
					timeStamps.add(string);
				}
				
				reader.readLine(); // "full backups"
				for (String string = reader.readLine(); !string.equals("."); string = reader.readLine()) {
					fullDep.add(string);
				}

				reader.readLine(); // "differential backups"
				for (String string = reader.readLine(); !string.equals("."); string = reader.readLine()) {
					diffDep.add(string);
				}

				reader.readLine(); // "incremental backups"
				for (String string = reader.readLine(); !string.equals("."); string = reader.readLine()) {
					incrDep.add(string);
				}

				reader.readLine(); // "list of all backups"
				for (String string = reader.readLine(); !string.equals("."); string = reader.readLine()) {
					list.add(string);
				}

				reader.close();
			} catch (Exception e) {
				Log.println("read dependencies failed for "+file);
				e.printStackTrace();
				throw new RuntimeException();
			}
		}
	}

	public String setTimeStamp (Settings settings) {
		if (finished) {
			// start a new backup set

			// check whether a base full backup is present or being made
			if (type == 0 && settings.backupType != 4) {			
				return "please perform a full backup of this directory\n"+
				  "before starting a backup of type "+settings.backupType;
			}
			
			// make up the new time stamp
			settings.timeStamp = Util.getTimeString() + "_01";
			
		} else {
			// continue or interrupt an existing backup set

			// if this backup has lower priority than an interrupted backup, stop backup
			if (type > settings.backupType) {
				return "first finish the backup of type "+type+"\n"+
							 "before starting a backup of type "+settings.backupType+"!";
			}

			// set backup time stamp, eventually derivated from earlier time stamp
			if (type < settings.backupType) {
				// interrupt old set by a new set of higher priority
				settings.timeStamp = Util.getTimeString() + "_01";
			} else {
				// get the time stamp to continue a set
				String base = settings.dependencies.timeStamps.get(0).toString();
				int count = Util.parseInt(base.substring(base.length()-2));
				settings.timeStamp = base.substring(0, base.length()-2) +
					new DecimalFormat("00").format(count+1);
			}
		}
		return "";
	}

	public void set (String path, Settings settings) {

		// eventually erase dependency chains of current backup type if a new set is begun
		if (finished) {
			switch (settings.backupType) {
				case 4:
				fullDep = new Vector();
				break;
				case 3:
				diffDep = new Vector();
				break;
			}
		}
		
		// update data
		switch (settings.backupType) {
			case 4:
			fullDep.add(settings.timeStamp);
			diffDep = new Vector();
			incrDep = new Vector();
			break;
			
			case 3:
			case 2:
			diffDep.add (settings.timeStamp);
			incrDep = new Vector();
			break;

			case 1:
			incrDep.add (settings.timeStamp);
			break;
			
			default:
			System.err.println("update dependencies failed");
			throw new RuntimeException();
		}
		list.add (0,settings.timeStamp);
		finished = settings.backupFinished;
		type = settings.backupType;

		timeStamps.add(0, settings.timeStamp);
		if (finished) {
			timeStamps = new Vector();
		}
		
		// write new file
		File file = new File(path+File.separator+".backup");
		try {

			BufferedWriter writer = new BufferedWriter(new FileWriter(file));

			writer.write("gb_1.1"); writer.newLine();
			writer.write("."); writer.newLine();

			writer.write("status, type, and, if incomplete, time stamps of the last backup set"); writer.newLine();
			writer.write(finished?"complete":"incomplete"); writer.newLine();
			writer.write(type+""); writer.newLine();
			for (Enumeration e = timeStamps.elements(); e.hasMoreElements(); ) {
				writer.write(e.nextElement().toString()); writer.newLine();
			}
			writer.write("."); writer.newLine();
			
			writer.write("full backup"); writer.newLine();
			for (Enumeration e = fullDep.elements(); e.hasMoreElements(); ) {
				writer.write(e.nextElement().toString()); writer.newLine();
			}
			writer.write("."); writer.newLine();

			writer.write("differential backups"); writer.newLine();
			for (Enumeration e = diffDep.elements(); e.hasMoreElements(); ) {
				writer.write(e.nextElement().toString()); writer.newLine();
			}
			writer.write("."); writer.newLine();

			writer.write("incremental backups"); writer.newLine();
			for (Enumeration e = incrDep.elements(); e.hasMoreElements(); ) {
				writer.write(e.nextElement().toString()); writer.newLine();
			}
			writer.write("."); writer.newLine();

			writer.write("list of all backups"); writer.newLine();
			for (Enumeration e = list.elements(); e.hasMoreElements(); ) {
				writer.write(e.nextElement().toString()); writer.newLine();
			}
			writer.write("."); writer.newLine();

			writer.close();

		} catch (Exception e) {
			System.err.println("write dependencies failes for "+file);
			e.printStackTrace();
			throw new RuntimeException();
		}
	}

}
