/*
 * jabak Java Backup
 * 
 * Copyright (C) under the terms of the GNU General Public License
 * as explained in the file "disclaimer.txt"
 */

package backup;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.text.NumberFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Enumeration;

import ui.Local;
import ui.Log;
import ui.StringContainer;

/**
 * a collection of static utility methods for the backup system
 * 
 * @author  Georg Dietrich
 * @version 1.3 on 2004-04-19
 * @version 1.0 on 2004-04-08
 */
public class Util {

	/**
	 * get a buffered reader, handle exceptions
	 * 
	 * @param file  the filepath to get the reader for
	 * @return      the buffered reader
	 */
	public static BufferedReader getReader (String file) {
		return getReader (new File(file));
	}

	/**
	 * get a buffered reader, handle exceptions
	 * 
	 * @param file  the file to get the reader for
	 * @return      the buffered reader
	 */
	public static BufferedReader getReader (File file) {
		try {
			return new BufferedReader(new FileReader (file));
		} catch (FileNotFoundException e) {
			Log.println("Exception "+e+" on getReader for file "+file);
			return null;
		}
	}

	/**
	 * read one line from a buffered reader, handle exceptions
	 * 
	 * @param reader  the reader to read from
	 * @return        the line read or null if end of stream is reached
	 */
	public static String readln (BufferedReader reader) {
		try {
			return reader.readLine();
		} catch (IOException e) {
			Log.println("Exception "+e+" on readLine for reader "+reader);
			return null;
		}
	}

	/**
	 * close a buffered reader, handle exceptions
	 * 
	 * @param reader  the reader to close
	 */
	public static void closeReader (BufferedReader reader) {
		try {
			reader.close();
		} catch (IOException e) {
			Log.println("Exception "+e+" on closeReader for "+reader);
		}
	}

	/**
	 * get the path the jabak executable is located
	 * 
	 * @return  the path the jabak executable is located
	 */
	public static String programPath () {
		// get the classpath
		File classPath = new File(System.getProperty("java.class.path"));
		// if classpath is a .jar file, take its parent directory
		if (!classPath.isDirectory()) classPath = classPath.getParentFile();
		return canonicalPath(classPath);
	}

	/** get the path of the data directory */
	public static String dataPath () {
		String result = programPath()+File.separator+"data";
		if (!new File(result).mkdirs()) {
			System.err.println("makedirs gescheitert fr "+result);
			throw new RuntimeException();
		}
		return result;
	}

	/** get the path of the help file */
	public static String helpPath () {
		return programPath()+File.separator+"help"+File.separator+"help.txt";
	}

	/** get the path of the default config file */
	public static String configFilePath () {
		return programPath()+File.separator+"jabak.config";
	}

	/** get the path of the log file */
	public static String logPath () {
		return programPath()+File.separator+Local.LOGFILE;
	}

	/**
	 * generate a hash sum out of a file's content
	 * 
	 * @param source  the source file
	 * @return        the hash sum
	 */
	public static long hashSum (File file) {
		try {
			
			long hash = 0;			
			FileInputStream filein  = new FileInputStream (file);
			byte[] buffer = new byte[1024];
			for (int read = filein.read(buffer); read != -1; read = filein.read(buffer)) {
				for (int i = 0; i < read; i ++) {
					hash ^= buffer[i]+255;
					if (hash > 1E10) hash -= 1E9;
				}
			}
			return hash;

		} catch (Exception e) {
			System.err.println("hashSum gescheitert fr "+file);
			e.printStackTrace();
			throw new RuntimeException();
		}
	}

	/**
	 * construct a backup file reference from a source reference
	 */
	public static File source2backup (File source, Settings settings) {
		String path = canonicalPath(source);
		return new File(settings.paths.backupTarget+path.substring(settings.paths.sourceDir.length())+".zip");
	}

	/**
	 * construct a data file reference from a source reference
	 */
	public static File source2data (File source, Settings settings) {
		String path = canonicalPath(source);
		return new File(settings.paths.dataDir+path.substring(settings.paths.sourceDir.length())+".txt");
	}

	/**
	 * construct a source file reference from a data reference
	 */
	public static File data2source (File data, Settings settings) {
		String path = canonicalPath(data);
		return new File(settings.paths.sourceDir+path.substring(settings.paths.dataDir.length(),path.length()-4));
	}

	/**
	 * parse a string into an int
	 * 
	 * @param string  the string to parse
	 * @return        the integer parsed
	 */
	public static int parseInt (StringContainer string) {
		return parseInt(string.toString());
	}

	/**
	 * parse a string into an int.  If parsing fails return -1.
	 * 
	 * @param string  the string to parse
	 * @return        the integer parsed or -1 on failure
	 */
	public static int parseInt (String string) {
		try {
			return NumberFormat.getInstance().parse(string).intValue();
		} catch (ParseException e) {
			return -1;
		}
	}

	/**
	 * parse a string into an long.  If parsing fails return -1.
	 * 
	 * @param string  the string to parse
	 * @return        the integer parsed or -1 on failure
	 */
	public static long parseLong (String string) {
		try {
			return NumberFormat.getInstance().parse(string).longValue();
		} catch (ParseException e) {
			return -1;
		}
	}

	/**
	 * recurses a directory tree performing actions on all files, not on dirs
	 * 
	 * @param target  the path to start with in the recursive search
	 */
	public static void recurseDirectory(File target, ObjectHandler handler) {
		if (target.isDirectory()) {
			File[] directoryContent = target.listFiles();
			for (int i = 0; i < directoryContent.length; i++) {
				recurseDirectory(directoryContent[i], handler);
			}
		} else {
			handler.perform(target);
		}
	}

	/**
	 * recurses a directory tree performing actions on all files, not on dirs
	 * but first filters file and dir names
	 * 
	 * @param target  the path to start with in the recursive search
	 * @return  false if recursion should be stopped
	 */
	public static boolean recurseDirectoryFilter(File target, ObjectHandler handler, Settings settings) {

		// filter
		String name = target.getName();
		for (Enumeration e = settings.fileFilters.elements(); e.hasMoreElements(); ) {
			if (name.matches(e.nextElement().toString()) ^ !settings.filterOut) return true;
		}
		
		// recurse / perform
		if (target.isDirectory()) {
			File[] directoryContent = target.listFiles();
			for (int i = 0; i < directoryContent.length; i++) {
				if (!recurseDirectoryFilter(directoryContent[i], handler, settings)) return false;
			}
			return true;
		} else {
			return handler.perform(target);
		}
	}

	/**
	 * get the canonical path for a path string
	 * 
	 * @param string  the path string to get the canonical path for
	 * @return        the canonical path of the corresponding file
	 */
	public static String canonicalPath (String string) {
		return canonicalPath(new File(string));
	}

	/**
	 * get the canonical path for a path string buffer
	 * 
	 * @param string  the path string to get the canonical path for
	 * @return        the canonical path of the corresponding file
	 */
	public static String canonicalPath (StringContainer buffer) {
		return canonicalPath(buffer.toString());
	}

	/**
	 * get the canonical path for a file
	 * 
	 * @param file  the file to get the canonical path for
	 * @return      the canonical path of the file
	 */
	public static String canonicalPath (File file) {
		try {
			return file.getCanonicalPath();
		} catch (IOException e) {
			System.err.println("canonicalPath gescheitert fr "+file);
			throw new RuntimeException();
		}
	}

	/**
	 * if the directories leading to an abstract file location do not exist,
	 * create them
	 * 
	 * @param file  the file location to create the directories for
	 */
	public static void makeDirs (File file) {
		if (file.getParentFile().exists()) {
			return;
		}
		if (!file.getParentFile().mkdirs()) {
			System.err.println("makedirs gescheitert fr "+file);
			throw new RuntimeException();
		}
	}
	
	/**
	 * get a string like 1972_12_10_0834 for the current time
	 * 
	 * @return  a time string for the current time
	 */
	public static String getTimeString () {
		return new SimpleDateFormat("yyyy_MM_dd_HHmm").format(Calendar.getInstance().getTime());
	}

	/**
	 * replace the contents of a string buffer by a string
	 * 
	 * @param buffer
	 * @param newString
	 * @return           a reference to the string buffer
	 */
	public static StringBuffer set (StringBuffer buffer, Object newString) {
		buffer.setLength(0);
		buffer.append(newString);
		return buffer;
	}
}
