/*
 * jabak Java Backup
 * 
 * Copyright (C) under the terms of the GNU General Public License
 * as explained in the file "disclaimer.txt"
 */

package ui;

import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JTabbedPane;

import backup.CoreMain;
import backup.Util;

/**
 * @author  Georg Dietrich
 * @version 2.1 on 2004-05-15
 * @version 2.0 on 2004-04-25
 * @version 1.2 on 2004-04-13
 * @version 1.1 on 2004-04-11
 * @version 1.0 on 2004-04-10
 */
public class GuiMain {

	/** 
	 * flag whether to issue a warning on closing the main window.
	 * this flag is set to true whenever a backup process is started
	 * and resetted to false when the backup process ends.
	 */
	static boolean runningWarning = false;

	/** the main application's main frame */
	static JFrame mainFrame = null;

	/** the progress bar */
	static JProgressBar progressBar = getProgressBar();

	/** the action object for disabling components */
	public static StringContainer disableRunning = new StringContainer();

	/** main application entry point */
	public static void main(String[] args) {

		/* evaluate settings from command line and config file */
		GuiSettings settings = new GuiSettings(args);

		// the main tabbed pane
		JTabbedPane tabs = new JTabbedPane(JTabbedPane.TOP);
		tabs.addTab("Main",getTab1(settings));
		tabs.addTab("Options",getTab2(settings));
		tabs.addTab("Log",new TextPane(Log.log));
		tabs.addTab("Help",new TextPane(new File(Util.helpPath())));

		// the main panel
		JPanel mainPane = new JPanel();
		mainPane.setLayout(new BoxLayout(mainPane, BoxLayout.Y_AXIS));
		mainPane.add(tabs);
		mainPane.add(new HintText("This will become a context-sensitive\nhelp & hint text field."));

		// the main frame
		mainFrame = new JFrame();
		mainFrame.setSize(720, 300);
		mainFrame.getContentPane().add(mainPane);
		mainFrame.setTitle("jabak Java Backup "+Local.VERSION);
		// I have my own close operation defined, see below
		mainFrame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		mainFrame.setVisible(true);
		
		// window exit routine issuing a warning if backup process is running
		mainFrame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				warnBeforeExiting();
			}
		});
	}

	/**
	 * first time called issues a warning.  Second time exits the program.
	 */
	static void warnBeforeExiting() {
		if (runningWarning) {
			runningWarning = false;
			JFrame text = new TextFrame ("Warning",
			  "By Closing this window you interrupt the\n"+
			  "currently running backup process.\n"+
				"If you interrupt a running backup process\n"+
				"the backup data become inconsistent and not\n"+
				"all files may be backed up as you intended.\n"+
				"If you really interrupt this backup process,\n"+
				"start over with a new full backup.");
			Point loc = mainFrame.getLocation();
			loc.y += mainFrame.getHeight();
			text.setLocation(loc);
		} else {
			System.exit(0);
		}
	}

	/**
	 * get the first content tab
	 * 
	 * @param settings
	 * @return
	 */
	private static JPanel getTab1(GuiSettings settings) {
		
		// create and layout panel (some rows, 3 columns, space between rows/cols, border)
		JPanel mainPane = new JPanel();
		mainPane.setLayout(new java.awt.GridLayout(0,3,6,6));

		// add content objects to the panel

		mainPane.add(new TextPanel("Source Direcory"));
		mainPane.add(disableRunning.addComponent(new TextField(settings.sourceDir)));
		mainPane.add(disableRunning.addComponent(new DirChooserButton("choose",settings.sourceDir)));

		mainPane.add(new TextPanel("Destination Directory"));
		mainPane.add(disableRunning.addComponent(new TextField(settings.backupDir)));
		mainPane.add(disableRunning.addComponent(new DirChooserButton("choose",settings.backupDir)));

		mainPane.add(new TextPanel("Backup Size (MB) and Type"));
		mainPane.add(disableRunning.addComponent(new TextField(settings.size)));
		mainPane.add(disableRunning.addComponent(new TypeChooser(Local.DROPLIST_TYPES, settings.backupType)));

		mainPane.add(new TextPanel("Progress"));
		mainPane.add(progressBar);
		mainPane.add(getStartButton(settings));

		return mainPane;
	}

	/**
	 * get the first content tab
	 * 
	 * @param settings
	 * @return
	 */
	private static JPanel getTab2(GuiSettings settings) {
		
		// create and layout panel (some rows, 3 columns, space between rows/cols, border)
		JPanel mainPane = new JPanel();
		mainPane.setLayout(new java.awt.GridLayout(0,3,6,6));

		// add content objects to the panel

		mainPane.add(new TextPanel("Data Directory:"));
		mainPane.add(disableRunning.addComponent(new TextField(settings.dataDir)));
		mainPane.add(disableRunning.addComponent(new DirChooserButton("choose",settings.dataDir)));

		mainPane.add(new TextPanel("File Filter"));
		mainPane.add(disableRunning.addComponent(new TextField(settings.filters)));
		mainPane.add(new TextPanel("'all' for all files"));

		mainPane.add(new TextPanel("Zip Filter Rules"));
		mainPane.add(disableRunning.addComponent(new TextField(settings.zipFilters)));
		mainPane.add(new TextPanel("0..9 compression factor"));

		mainPane.add(new TextPanel("change detection"));
		mainPane.add(new TypeChooser(Local.COMP_TYPE_NAMES, settings.compare));
		mainPane.add(new TextPanel(""));

		// TODO: implement File Saver
		mainPane.add(new TextPanel("Save current settings"));
		// TODO: implement JToggleButton
		mainPane.add(new TextPanel(""));
		// TODO: implement stop button
		mainPane.add(new TextPanel("Stop"));
		
		return mainPane;
	}



	/**
	 * @return  a help button that displays the help file
	 */
	private static JButton getHelpButton() {

		// create the help button
		JButton help = new JButton();
		help.setText("Help");

		// set up the help action
		help.addActionListener(new java.awt.event.ActionListener() { 
			public void actionPerformed(java.awt.event.ActionEvent e) {

				// find the help directory
				String path = System.getProperty("java.class.path");
				File pFile = new File (path);
				if (!pFile.isDirectory()) pFile = pFile.getParentFile();
				path = Util.canonicalPath(pFile);
				
				// display the help file
				JFrame text = new TextFrame("Help me!", new File(path+File.separator+"help"+File.separator+"help.txt"));
				Point loc = mainFrame.getLocation();
				loc.y += mainFrame.getHeight();
				text.setLocation(loc);
			}
		});

		return help;
	}

	/**
	 * @return  a percentile progress bar with text display
	 */
	private static JProgressBar getProgressBar() {
		JProgressBar bar = new JProgressBar(0,100);
		bar.setStringPainted(true);
		bar.setString("0 skipped / 0 backed up");
		return bar;
	}

	/**
	 * the start button is a multifunctional button that
	 * changes button text and function according to the
	 * program state:
	 * initially the button serves for starting the
	 * backup process.  during backup the button serves
	 * to interrupt the backup (same functionality as
	 * the close button on the upper right corner).
	 * after backup the button becomes an ordinary
	 * close button.
	 * 
	 * @param settings
	 * @return
	 */
	private static JButton getStartButton(final GuiSettings settings) {
		
		// create the button
		final JButton button = new JButton();
		button.setText("start");
		
		// add action listener listening for button activation
		button.addActionListener(new ActionListener() { 
			public void actionPerformed(ActionEvent e) {
				
//				// if backup process is finished, exit
//				if (button.getText().equals("Finished - Exit")) {
//					System.exit(0);
//				}
				
				// if backup process is running, warn before exiting
				if (button.getText().equals("Running - Interrupt")) {
					warnBeforeExiting();
					return;
				}

				// check whether to start backup process
				if (settings.check()) {

					// redirect button use
					runningWarning = true;
					button.setText("Running - Interrupt");
					
					// disable all settings fields and buttons
					disableRunning.setString("disabled");

					// enable the graphical progress interface			
					// TODO: set up configurable log value
					settings.log = new ProgressInterface(progressBar, 2);

					// set up new swing worker for the backup process
					SwingWorker worker = new SwingWorker () {

						public Object construct() {
							// construct the backup core
							new CoreMain(settings);
							return null;
						}

						public void finished () {

							// finished - re-enable the start button
							button.setText("start again");
							runningWarning = false;

							// enable all settings fields and buttons
							disableRunning.setString("enabled");

							// display finishing message
							JFrame text = new TextFrame ("Backup result", settings.result);
							Point loc = mainFrame.getLocation();
							loc.y += mainFrame.getHeight();
							text.setLocation(loc);
						}
					};
					// start backup process
					worker.start();
				}
			}
		});
		return button;
	}
	
}
