/*
 * jabak Java Backup
 * 
 * Copyright (C) under the terms of the GNU General Public License
 * as explained in the file "disclaimer.txt"
 */

package ui;

import java.io.File;

import backup.Util;

/**
 * basic rule class to check whether strings are formatted the right way,
 * and to reformat strings according to rules, and a collection of rules
 * 
 * @author  Georg Dietrich
 * @version 1.2 on 2004-05-16
 * @version 1.1 on 2004-05-05
 * @version 1.0 on 2004-04-20
 */
public abstract class Rule {

	/**
	 * check whether a string follows a the rule.  Implemented by subclasses.
	 * 
	 * @param string  the string to check
	 * @return        true if rule is followed
	 */	
	abstract boolean check(String string);
	
	/**
	 * convert an object into a string and check the string
	 * 
	 * @param object  the object to check
	 * @return        true if rule is followed
	 */
	boolean check(Object object)  { return check(object.toString()); }

	/**
	 * reformat a string according to a rule
	 * 
	 * @param string  the string to reformat
	 * @return        the reformatted string
	 */
	String  format(String string) { return string; }

	// ************ rule collection below ***************

	/** rule that always returns true */
	public static final Rule TRUE = new Rule () {
		boolean check (String string) { return true; }
	};

	/**
	 * rule checking for the key words "hash", "date", or "both", or for the
	 * equivalent numbers 1-3, signalling how to detect changes in source files
	 */
	public static final Rule COMPARE = new Rule() {
		boolean check (String string) {
			// detect the numbers 1-3
			if (Util.parseInt(string) >= 0 && Util.parseInt(string) < Local.COMP_TYPES.size()) return true;
			// detect the key words
			return Local.COMP_TYPES.contains(string);
		}
		/** replace key words through key numbers in the string */
		String format (String string) {
			int position = Local.COMP_TYPES.indexOf(string);
			if (position != -1) return String.valueOf(position);
			return string;
		}
	};

	/** rule for checking whether a String denotes a directory */
	public static final Rule DIRECTORY = new Rule () {
		boolean check (String string) { return new File(string).isDirectory(); }
	};
	
	/** rule for checking whether a String denotes an existing file */
	public static final Rule FILE = new Rule () {
		boolean check (String string) { return new File(string).exists(); }
	};
	
	/** rule for checking whether a String denotes a size */
	public static final Rule SIZE = new Rule () {
		boolean check (String string) { return Util.parseInt(string) > 0; }
	};

	/** rule for checking whether a String denotes a backup type */
	public static final Rule TYPE = new Rule () {
		boolean check (String string) {
			// check for number (1..4)
			if (Util.parseInt(string) >= 0 && Util.parseInt(string) < Local.BACKUP_TYPES.size()) return true;
			// check for type name
			return Local.BACKUP_TYPES.contains(string);
		}
		/** replace key words through key numbers in the string */
		String format (String string) {
			int position = Local.BACKUP_TYPES.indexOf(string);
			if (position != -1) return String.valueOf(position);
			return string;
		}
	};

	/** rule for checking whether a String denotes valid filter rule */
	public static final Rule FILTER = new Rule () {
		boolean check (String string) {
			// TODO: move implementation here
			return GuiSettings.filterCheck(string);
		}
	};

	/** rule for checking whether a String denotes valid zip filter rule */
	public static final Rule ZIPFILTER = new Rule () {
		boolean check (String string) {
			// TODO: move implementation here
			return GuiSettings.zipFilterCheck(string);
		}
	};

}
