/*
 * jabak Java Backup
 * 
 * Copyright (C) under the terms of the GNU General Public License
 * as explained in the file "disclaimer.txt"
 */

package ui;

import java.io.BufferedReader;
import java.io.File;
import java.util.Arrays;
import java.util.Vector;

import backup.Util;

/**
 * This classes static methods serve for parsing command line args
 * and settings files into the gui settings object.
 * 
 * @author  Georg Dietrich
 * @version 1.2 on 2004-04-29
 * @version 1.0 on 2004-04-18
 */
public class SettingsParser {

	/**
	 * scan command line arguments and settings files and set gui settings appropriately
	 * 
	 * @param arg  the command line arguments
	 */
	public static void parseSettings (String[] arg, GuiSettings settings) {

		// parse the command line settings
		StringContainer configfile = new StringContainer();
		boolean useDefaultConfigFile = parseCmdLineSettings(arg, settings, configfile);
		
		// eventually parse user config file
		File file = new File (configfile.toString());
		if (file.exists()) parseConfigFile(file, settings);
		
		// eventually parse default config file
		file = new File (Util.configFilePath());
		if (useDefaultConfigFile && file.exists()) parseConfigFile(file, settings);
		
		// set default settings for some settings not yet set correctly

		// set data dir to default
		if (settings.dataDir.toString().equals("")) {
			settings.dataDir.setString(Util.dataPath());
		}
		
		// set file filter to default
		if (settings.filters.toString().equals("")) {
			settings.filters.setString("all");
		}
	}

	/**
	 * scan a config file and set all parameters found unless they are already set
	 * by earlier scans of command line or config file
	 * 
	 * @param file
	 * @param settings
	 */	
	protected static void parseConfigFile (File file, GuiSettings settings) {

		// read file line by line into vector
		BufferedReader reader = Util.getReader(file);
		Vector lines = new Vector();
		for (String string = Util.readln(reader); string != null; string = Util.readln(reader)) lines.add(string);
		Util.closeReader(reader);
		
		// parse the single switches
		scanFile (Local.SOURCE_SWITCH, lines, settings.sourceDir, Rule.DIRECTORY);
		scanFile (Local.DEST_SWITCH, lines, settings.backupDir, Rule.DIRECTORY);
		scanFile (Local.DATA_SWITCH, lines, settings.dataDir, Rule.DIRECTORY);
		scanFile (Local.SIZE_SWITCH, lines, settings.size, Rule.SIZE);
		scanFile (Local.FILTER_SWITCH, lines, settings.filters, Rule.FILTER);
		scanFile (Local.ZIP_SWITCH, lines, settings.zipFilters, Rule.ZIPFILTER);
		scanFile (Local.TYPE_SWITCH, lines, settings.backupType, Rule.TYPE);
		scanFile (Local.COMP_SWITCH, lines, settings.compare, Rule.COMPARE);
		// TODO: continue implementation

	}

	/**
	 * scan a config file for a specific switch and check options rules
	 * 
	 * @param find     the switches to look for
	 * @param args     the command line args to look in
	 * @param target   the target to write options into or null if no options expected
	 * @param rule     the rule to check the options' validity (the TRUE rule for no options)
	 * @return         true if the switch was found and its parameters are valid
	 */
	protected static boolean scanFile (String[] find, Vector lines, StringContainer target, Rule rule) {
		// TODO: zusammenfassen mit scanCmdLine
		// find the switch and the optional parameter
		String param = scanFile (find, lines);
		if (param != null) {
			// check the parameter found
			if (rule.check(param)) {
				// everything o.k.
				if (target == null) return true; // no parameters expected
				// only set param if not already set; eventually bring param into the right format
				if (target.length() == 0) target.setString(rule.format(param));
				return true;
			} else {
				// wrong argument
				Log.println("bad config file option "+param+" after -"+Arrays.asList(find));
				return false;
			}
		}
		// no switch
		return false;
	}

	/**
	 * scan settings file for specific switch and return its option as string
	 * 
	 * @param find   the strings that indicate the switch
	 * @param lines  the lines of the file
	 * @return       a option string, "" if no option is found, or null if switch was not found
	 */
	protected static String scanFile (String[] find, Vector lines) {
		// look for every command line switch from the find field
		for (int i = 0; i < find.length; i++) {
			// check whether the current switch is in the args vector
			if (lines.contains(Local.SWITCH_IND+find[i])) {
				// fetch the index the switch is found at
				int pos = lines.indexOf(Local.SWITCH_IND+find[i]);
				// return argument line or "" at end of file
				if (lines.size() > pos+1) {
					return (String)lines.get(pos+1);
				} else {
					return "";
				}
			}
		}
		// switch was not found
		return null;
	}

	/**
	 * scan command line arguments and set gui settings appropriately
	 * 
	 * @param arg         the command line arguments
	 * @param settings    the gui settings to set
	 * @param configfile  the optional user config file specified on command line
	 * @return            true if default configfile should be parsed
	 */
	protected static boolean parseCmdLineSettings (String[] arg, GuiSettings settings, StringContainer configfile) {

		// make vector out of command line args
		Vector args =  new Vector(Arrays.asList(arg));

		// parse command line args
		
		// -help
		if (scanCmdLine (Local.HELP_SWITCH, args, null, Rule.TRUE)) {
			System.out.println(Local.SHORT_HELP);
			System.exit(0);
		}
		
		// -longhelp
		if (scanCmdLine (Local.LONGHELP_SWITCH, args, null, Rule.TRUE)) {
			BufferedReader reader = Util.getReader(Util.helpPath());
			for (String read = Util.readln(reader); read != null; read = Util.readln(reader)) System.out.println(read);
			System.exit(0);
		}

		// -version
		if (scanCmdLine (Local.VERSION_SWITCH, args, null, Rule.TRUE)) {
			System.out.println(Local.VERSION_DISCLAIMER);
			System.exit(0);
		}
		
		// settings switches
		scanCmdLine (Local.SOURCE_SWITCH, args, settings.sourceDir,     Rule.DIRECTORY);
		scanCmdLine (Local.DEST_SWITCH,   args, settings.backupDir, Rule.DIRECTORY);
		scanCmdLine (Local.DATA_SWITCH,   args, settings.dataDir,   Rule.DIRECTORY);
		scanCmdLine (Local.SIZE_SWITCH,   args, settings.size,       Rule.SIZE);
		scanCmdLine (Local.TYPE_SWITCH,   args, settings.backupType, Rule.TYPE);
		scanCmdLine (Local.FILTER_SWITCH, args, settings.filters,    Rule.FILTER);
		scanCmdLine (Local.ZIP_SWITCH,    args, settings.zipFilters, Rule.ZIPFILTER);
		scanCmdLine (Local.COMP_SWITCH, args, settings.compare, Rule.COMPARE);


		if (scanCmdLine (Local.NOGUI_SWITCH,     args, null, Rule.TRUE)) settings.noGui = true;
		if (scanCmdLine (Local.IMMEDIATE_SWITCH, args, null, Rule.TRUE)) settings.immediateStart = true;
		
		scanCmdLine (Local.CONFFILE_SWITCH, args, configfile, Rule.FILE);

		boolean noDefaultConfigFile = scanCmdLine (Local.NOCONFFILE_SWITCH, args, null, Rule.TRUE);
		
		// check whether we have leftover command line arguments
		if (args.size() > 0) {
			Log.println("unable to process command line arguments "+args);		
		}
		
		return !noDefaultConfigFile;
	}

	/**
	 * scan command line for a specific switch and check options rules
	 * 
	 * @param find     the switches to look for
	 * @param args     the command line args to look in
	 * @param target   the target to write options into or null if no options expected
	 * @param rule     the rule to check the options' validity
	 * @return         true if the switch was found and its parameters are valid
	 */
	protected static boolean scanCmdLine (String[] find, Vector args, StringContainer target, Rule rule) {
		// find the switch and the optional parameter
		String param = scanCmdLine (find, args);
		if (param != null) {
			// check the parameter found
			if (rule.check(param)) {
				// everything o.k.
				if (target == null) return true;  // no options expected
				// set option
				target.setString(param);
				return true;
			} else {
				// wrong option
				Log.println("bad option "+param+" after -"+Arrays.asList(find));
				return false;
			}
		}
		// no switch
		return false;
	}

	/**
	 * scan arguments for specific switch and return its option as string,
	 * removing switch and option from the arguments vector
	 * 
	 * @param find  the strings that indicate the switch
	 * @param args  the current set of arguments to scan
	 * @return      a option string, "" if no option is found, or null if switch was not found
	 */
	protected static String scanCmdLine (String[] find, Vector args) {
		// look for every command line switch from the find field
		for (int i = 0; i < find.length; i++) {
			// check whether the current switch is in the args vector
			if (args.contains(Local.SWITCH_IND+find[i])) {
				// fetch the index the switch is found at
				int pos = args.indexOf(Local.SWITCH_IND+find[i]);
				// remove switch
				args.remove(pos);
				// return argument or ""
				if (args.size() > pos && !args.get(pos).toString().startsWith(Local.SWITCH_IND)) {
					return (String)args.remove(pos);
				} else {
					return "";
				}
			}
		}
		// switch was not found
		return null;
	}
}
